
// 
// NUnitReportTask.cs
//    Loosely based on Erik Hatcher JUnitReport for Ant.
//
// Author:
//    Tomas Restrepo (tomasr@mvps.org)
//
// Copyright (C) 2002 Tomas Restrepo
//

//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

using System;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Text;
using System.Web.Mail;
using System.Xml;
using System.Xml.Xsl;
using SourceForge.NAnt.Attributes;
using SourceForge.NAnt;

namespace Winterdom.NAnt.Tasks 
{ 

   /// <summary>
   /// A task that generates a summary HTML
   /// from a set of NUnit xml report files.
   /// </summary>
   /// <remarks>
   /// This task can generate a combined HTML report out of a
   /// set of NUnit result files generated using the 
   /// XML Result Formatter.
   /// 
   /// By default, NUnitReport will generate the combined
   /// report using the NUnitSummary.xsl file located at the
   /// assembly's location, but you can specify a different
   /// XSLT template to use with the <code>xslfile</code>
   /// attribute.
   /// 
   /// Also, all the properties defined in the current
   /// project will be passed down to the XSLT file as 
   /// template parameters, so you can access properties
   /// such as nant.project.name, nant.version, etc.
   /// </remarks>
   /// <example>
   ///   <code><![CDATA[
   ///   <nunitreport 
   ///         out="${outputdir}\TestSummary.html"
   ///         >
   ///      <fileset>
   ///         <includes name="${outputdir}\Test-*.xml" />
   ///      </fileset>
   ///   </nunitreport>
   ///   
   ///   ]]></code>
   /// </example>
   [TaskName("nunitreport")]
   public class NUnitReportTask : Task
   {
      private const string XSL_DEF_FILE = "NUnitSummary.xsl";

      private string _outFilename;
      private FileSet _fileset = new FileSet();
      private string _xsl;

      /// <summary>
      /// Name of Output HTML file.
      /// </summary>
      [TaskAttribute("out", Required=true)]
      public string OutFilename { 
         get { return _outFilename; } 
         set { _outFilename = value; } 
      }
        
      /// <summary>
      /// XSLT file used to generate the report.
      /// </summary>
      [TaskAttribute("xslfile", Required=false)]
      public string XslFile { 
         get { return _xsl; } 
         set { _xsl = value; } 
      }

      /// <summary>
      /// Set of XML files to use as input
      /// </summary>
      [FileSet("fileset")]
      public FileSet XmlFileSet {
         get { return _fileset; }
      }


      ///<summary>
      ///Initializes task and ensures the supplied attributes are valid.
      ///</summary>
      ///<param name="taskNode">Xml node used to define this task instance.</param>
      protected override void InitializeTask(System.Xml.XmlNode taskNode) 
      {
         if (OutFilename == null) {
            throw new BuildException("NUnitReport attribute \"out\" is required.", Location);
         }
         if (XslFile == null) {
            Assembly thisAssm = Assembly.GetExecutingAssembly();
            string location = Path.GetDirectoryName(thisAssm.Location);
            XslFile = Path.Combine(location, XSL_DEF_FILE);
         }

         if (XmlFileSet.FileNames.Count == 0) {
            throw new BuildException("NUnitReport fileset cannot be empty!", Location);
         }

      }

      /// <summary>
      /// This is where the work is done
      /// </summary>
      protected override void ExecuteTask() 
      {
         XmlDocument summary = CreateSummaryXmlDoc();

         foreach ( string file in XmlFileSet.FileNames ) {
            XmlDocument source = new XmlDocument();
            source.Load(file);
            XmlNode node = summary.ImportNode(source.DocumentElement, true);
            summary.DocumentElement.AppendChild(node);
         }

         //
         // prepare properties and transform
         //
         XsltArgumentList args = GetPropertyList();

         XslTransform xslt = new XslTransform();
         xslt.Load(XslFile);
         XmlTextWriter writer = new XmlTextWriter(OutFilename, null);
         xslt.Transform(summary, args, writer);
         writer.Close();
      }


      /// <summary>
      /// Initializes the XmlDocument instance
      /// used to summarize the test results
      /// </summary>
      /// <returns></returns>
      private XmlDocument CreateSummaryXmlDoc()
      {
         XmlDocument doc = new XmlDocument();
            XmlElement root = doc.CreateElement("testsummary");
            root.SetAttribute("created", DateTime.Now.ToString());
            doc.AppendChild(root);
         return doc;
      }

      /// <summary>
      /// Builds an XsltArgumentList with all
      /// the properties defined in the 
      /// current project as XSLT parameters.
      /// </summary>
      /// <returns></returns>
      private XsltArgumentList GetPropertyList()
      {
         XsltArgumentList args = new XsltArgumentList();

         foreach ( DictionaryEntry entry in this.Project.Properties )
         {
            args.AddParam((string)entry.Key, "", (string)entry.Value);
         }
         return args;
      }

   } // class NUnitReportTask

} // namespace Winterdom.NAnt.Tasks
